@@ -6,7 +6,7 @@ c=======================================================================
      &                      maxm,ntprob,ntsets,                             
      &                      a0b0,b0,prec,sb,tau1,tau2,m,s,             
      &                      mcmc,nsave,algo,              
-     &                      acrate,cpo,
+     &                      acrate,cpo,cpov,
      &                      densave,cdfsave,randsave,thetasave,bf,    
      &                      alpha,b,beta,mu,sigma2,probmat, 
      &                      seed,         
@@ -257,6 +257,7 @@ c+++++MCMC parameters
 c+++++Output
       real*8 acrate
       real*8 cpo(nsubject,p)
+      real*8 cpov(nsubject)
       real*8 densave(nsave,ngrid),cdfsave(nsave,ngrid)
       real*8 randsave(nsave,nsubject+1)
       real*8 thetasave(nsave,p+4)
@@ -1178,8 +1179,8 @@ c+++++++++++++ density
 
 
 c+++++++++++++ cpo
-
                do i=1,nsubject
+                  tmp2=0.d0
                   do j=1,p
                      yij=y(i,j)
                      if(j.eq.1)then
@@ -1188,9 +1189,11 @@ c+++++++++++++ cpo
                        eta=b(i)-beta(j-1)+roffset(i,j)
                      end if  
                      mean=exp(eta)
-                     tmp1=dpoiss(dble(yij),mean,0)
-                     cpo(i,j)=cpo(i,j)+1.d0/tmp1   
+                     tmp1=dpoiss(dble(yij),mean,1)
+                     cpo(i,j)=cpo(i,j)+1.d0/exp(tmp1)
+                     tmp2=tmp2+tmp1    
                   end do
+                  cpov(i)=cpov(i)+1.d0/exp(tmp2)
                end do
 
 c+++++++++++++ Savage-Dickey ratio
@@ -1247,7 +1250,8 @@ c+++++++++++++ print
       do i=1,nsubject
          do j=1,p
             cpo(i,j)=dble(nsave)/cpo(i,j)
-         end do   
+         end do 
+         cpov(i)=dble(nsave)/cpov(i)  
       end do
 
       numerBF=numerBF/dble(nsave)
